Imports Spread.Common.DataStore.Models
Imports Newtonsoft.Json
Imports Newtonsoft.Json.Converters
Imports System.Collections.Generic
Imports System.Linq
Imports Spread.Common.DataStore.Utilities
Imports Spread.Common.DataStore.Spread.Common.DataStore.Models

Public Class JsonDataSource

#Region "Fields"
    Private Const _jsonPath As String = ".\Data"
#End Region

#Region "ctor"
    Friend Sub New()
    End Sub
#End Region

#Region "Private methods"
    Private Function GetJSonFromFile(filePath As String) As String
        Return FileUtilities.ReadTextFromFile(filePath)
    End Function

    Private Sub UpdateDateInMatchData(matchListCurrent As List(Of Match), matchListPrevious As List(Of Match), selectedYr As String)
        Dim year As Integer = 0
        Dim monthStart As Integer = 0
        Dim dayStart As Integer = 0
        Dim monthLast As Integer = 0
        Dim dayLast As Integer = 0
        Dim gen As New Random()
        Dim start As DateTime = DateTime.Now
        Dim range As Integer = 0
        Dim selectedYrs As String() = selectedYr.Split("-"c)
        Dim yearSelected As Integer = If(selectedYrs.Count() < 2, Convert.ToInt32(selectedYrs(0)), Convert.ToInt32(selectedYrs(1)))

        If yearSelected = DateTime.Now.Year Then
            year = yearSelected
            monthStart = 1
            dayStart = 1
            monthLast = DateTime.Now.Month
            dayLast = DateTime.Now.Day
            start = New DateTime(year, monthStart, dayStart)

            range = (New DateTime(year, monthLast, dayLast) - start).Days
        Else
            year = yearSelected
            monthStart = 1
            dayStart = 1
            monthLast = 12
            dayLast = 31
            start = New DateTime(year, monthStart, dayStart)

            range = (New DateTime(year, monthLast, dayLast) - start).Days
        End If
        For Each match As Match In matchListCurrent
            match.MatchDate = RandomDay(gen, range, start)
        Next
        year = yearSelected - 1
        monthStart = 1
        dayStart = 1
        monthLast = 12
        dayLast = 31
        start = New DateTime(year, monthStart, dayStart)
        range = (New DateTime(year, monthLast, dayLast) - start).Days

        For Each match As Match In matchListPrevious
            match.MatchDate = RandomDay(gen, range, start)
        Next
    End Sub

    Private Function RandomDay(gen As Random, range As Integer, start As DateTime) As DateTime
        Return start.AddDays(gen.[Next](range))
    End Function
#End Region

#Region "Public methods"

    ''' <summary>
    ''' Get a List of all Customers
    ''' </summary>
    ''' <returns>List of all Customers</returns>
    Public Function GetAllCustomers() As List(Of Customer)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Customers.json"))
        Dim customers As List(Of Customer) = JsonConvert.DeserializeObject(Of List(Of Customer))(json)
        Return customers
    End Function
    Public Function GetAgingReportCustomers() As List(Of Customer)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "AgingReportCustomers.json"))
        Dim customers As List(Of Customer) = JsonConvert.DeserializeObject(Of List(Of Customer))(json)
        Return customers
    End Function

    Public Function GetAllParts() As List(Of Parts)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Parts.json"))
        Dim parts As List(Of Parts) = JsonConvert.DeserializeObject(Of List(Of Parts))(json)

        Return parts
    End Function
    Public Function GetAllInvoiceIDs() As List(Of String)
        Dim invoices As List(Of Invoice) = GetAllInvoices()
        Return invoices.[Select](Function(inv) inv.ID).ToList()
    End Function
    Public Function GetAllInvoices() As List(Of Invoice)
        Dim json As String = String.Empty
        Try
            json = FileUtilities.LoadJSON("Invoice.json")
        Catch
            json = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Invoice.json"))
        End Try
        Dim invoices As List(Of Invoice) = JsonConvert.DeserializeObject(Of List(Of Invoice))(json)
        Return invoices
    End Function
    Public Function GetAgingReportInvoices() As List(Of Invoice)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "AgingReportInvoices.json"))
        Dim invoices As List(Of Invoice) = JsonConvert.DeserializeObject(Of List(Of Invoice))(json)

        Return invoices
    End Function
    Public Function GetAgingReportInvoices(startDate As DateTime, endDate As DateTime) As List(Of Invoice)
        Dim random As New Random()
        Dim invoices As List(Of Invoice) = GetAgingReportInvoices()
        invoices = invoices.Where(Function(inv) inv.CreatedOn >= startDate AndAlso inv.CreatedOn <= endDate).OrderBy(Function(inv) random.[Next]()).ToList()
        Dim count As Integer = random.[Next](Math.Min(60, invoices.Count), 300)
        invoices = invoices.Take(Math.Min(count, invoices.Count)).ToList()

        ' Update 'ID' property of Invoice to be increasing
        Dim id As Integer = 101
        For i As Integer = 0 To invoices.Count - 1
            invoices(i).ID = System.Math.Max(System.Threading.Interlocked.Increment(id), id - 1).ToString()
        Next

        Return invoices.ToList()
    End Function

    Public Function GetCustomer(id As Integer) As Customer
        Dim customers As List(Of Customer) = GetAllCustomers()
        Return customers.Where(Function(cust) cust.ID = id).FirstOrDefault()
    End Function
    Public Function GetVehicle(id As String) As Vehicle
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Vehicle.json"))
        Dim vehicles As List(Of Vehicle) = JsonConvert.DeserializeObject(Of List(Of Vehicle))(json)

        Return vehicles.Where(Function(v) v.ID = id).FirstOrDefault()
    End Function
    Public Function GetInvoice(id As String) As Invoice
        Dim invoices As List(Of Invoice) = GetAllInvoices()
        Return invoices.Where(Function(inv) inv.ID = id).FirstOrDefault()
    End Function
    Public Function GetParts(id As String) As Parts
        Dim parts As List(Of Parts) = GetAllParts()
        Return parts.Where(Function(p) p.ID = id).FirstOrDefault()
    End Function
    Public Function GetInvoiceParts(invoiceID As String) As List(Of InvoiceParts)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "InvoiceParts.json"))
        Dim invoiceParts As List(Of InvoiceParts) = JsonConvert.DeserializeObject(Of List(Of InvoiceParts))(json)
        Return invoiceParts.Where(Function(invP) invP.InvoiceID = invoiceID).ToList()
    End Function
    Public Function GetInvoiceServices(invoiceID As String) As List(Of InvoiceServices)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "InvoiceServices.json"))
        Dim invoiceServices As List(Of InvoiceServices) = JsonConvert.DeserializeObject(Of List(Of InvoiceServices))(json)

        Return invoiceServices.Where(Function(invS) invS.InvoiceID = invoiceID).ToList()
    End Function
    Public Sub UpdateInvoice(invoice As Invoice)
        Dim invoices As List(Of Invoice) = GetAllInvoices()
        Dim newInvoice As Invoice = invoices.Where(Function(inv) inv.ID = invoice.ID).FirstOrDefault()
        newInvoice.TotalPartsAmount = invoice.TotalPartsAmount
        newInvoice.TotalServiceAmount = invoice.TotalServiceAmount
        newInvoice.TotalAmount = invoice.TotalAmount
        Try
            System.IO.File.WriteAllText(System.IO.Path.Combine(_jsonPath, "Invoice.json"), JsonConvert.SerializeObject(invoices, Formatting.Indented))
        Catch
        End Try
    End Sub
    Public Function GetInvoiceByCustomerID(customerID As Integer) As List(Of Invoice)
        Dim invoices As List(Of Invoice) = GetAllInvoices()
        Return invoices.Where(Function(inv) inv.CustomerID = customerID).ToList()
    End Function
    Public Function GetInvoiceByVehicleID(vehicleID As String) As List(Of Invoice)
        Dim invoices As List(Of Invoice) = GetAllInvoices()
        Return invoices.Where(Function(inv) inv.VehicleID = vehicleID).ToList()
    End Function
    Public Function GetInvoiceByCustomerName(customerName As String) As List(Of Invoice)
        Dim json As String = System.IO.File.ReadAllText(System.IO.Path.Combine(_jsonPath, "Customer.json"))
        Dim customerList As List(Of Customer) = JsonConvert.DeserializeObject(Of List(Of Customer))(json)
        Dim invoiceList As List(Of Invoice) = GetAllInvoices()

        Dim customers As List(Of Customer) = customerList.Where(Function(cust) cust.Name = customerName).ToList()

        Dim invoices As New List(Of Invoice)()
        For Each customer As Customer In customers
            invoices.AddRange(invoiceList.Where(Function(inv) inv.CustomerID = customer.ID).AsEnumerable())
        Next
        Return invoices
    End Function

    Public Function GetAllTeams() As List(Of Team)
        Dim _jsonTeamStr As String = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "Team.json"))
        Dim format = "dd/MM/yyyy"
        ' your datetime format
        Dim dateTimeConverter = New IsoDateTimeConverter() With {
            .DateTimeFormat = format
        }
        Dim teams As List(Of Team) = JsonConvert.DeserializeObject(Of List(Of Team))(_jsonTeamStr, dateTimeConverter)

        Return teams
    End Function

    Public Function GetCurrentAndPreviousMatches(range As Integer, selectedYr As String) As List(Of Match)()
        Dim currFileName As String = ""
        Dim prevFileName As String = ""
        Select Case DateTime.Now.Year - range
            Case 0
                currFileName = "MatchCurrentYear.json"
                prevFileName = "MatchLastYear.json"
                Exit Select
            Case 1
                currFileName = "MatchLastYear.json"
                prevFileName = "MatchSecondLastYear.json"
                Exit Select
            Case 2
                currFileName = "MatchSecondLastYear.json"
                prevFileName = "MatchThirdLastYear.json"
                Exit Select
            Case 3
                currFileName = "MatchThirdLastYear.json"
                prevFileName = "MatchFourthLastYear.json"
                Exit Select
            Case 4
                currFileName = "MatchFourthLastYear.json"
                prevFileName = ""
                Exit Select
            Case Else
                Exit Select
        End Select
        Dim _jsonMatchStr As String = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, currFileName))
        Dim format = "dd/MM/yyyy"
        ' your datetime format
        Dim dateTimeConverter = New IsoDateTimeConverter() With {
            .DateTimeFormat = format
        }

        Dim matchListCurrent As List(Of Match) = JsonConvert.DeserializeObject(Of List(Of Match))(_jsonMatchStr, dateTimeConverter)
        Dim _jsonMatchStrPre As String = Nothing
        If prevFileName <> "" Then
            _jsonMatchStrPre = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, prevFileName))
        End If

        Dim matchListPrevious As List(Of Match) = Nothing
        If _jsonMatchStrPre IsNot Nothing Then
            matchListPrevious = JsonConvert.DeserializeObject(Of List(Of Match))(_jsonMatchStrPre, dateTimeConverter)
        Else
            matchListPrevious = New List(Of Match)()
        End If
        UpdateDateInMatchData(matchListCurrent, matchListPrevious, selectedYr)

        Dim matches As List(Of Match)() = New List(Of Match)() {matchListCurrent, matchListPrevious}
        Return matches
    End Function

    Public Function GetAllEmployees() As List(Of Employee)
        Dim jsonEmpStr As String = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "Employee.json"))
        Dim format = "dd/MM/yyyy"
        ' your datetime format
        Dim dateTimeConverter = New IsoDateTimeConverter() With {
           .DateTimeFormat = format
        }
        Dim employees As List(Of Employee) = JsonConvert.DeserializeObject(Of List(Of Employee))(jsonEmpStr, dateTimeConverter)

        Return employees
    End Function

    Public Function GetAbsenceScheduleList() As List(Of AbsenceSchedule)
        Dim jsonAbsenceStr As String = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "AbsenceSchedule.json"))
        Dim format = "dd/MM/yyyy"
        ' your datetime format
        Dim dateTimeConverter = New IsoDateTimeConverter() With {
            .DateTimeFormat = format
        }
        Dim absenceSchedule As List(Of AbsenceSchedule) = JsonConvert.DeserializeObject(Of List(Of AbsenceSchedule))(jsonAbsenceStr, dateTimeConverter)
        Return absenceSchedule
    End Function

    Public Function GetMovieRecords() As List(Of MovieRecord)
        Dim jsonMovieStr As String = GetJSonFromFile(System.IO.Path.Combine(_jsonPath, "movie_data.json"))
        Dim movieRecords As List(Of MovieRecord) = JsonConvert.DeserializeObject(Of List(Of MovieRecord))(jsonMovieStr)
        Return movieRecords
    End Function

    Public Sub SaveObjectToJsonFile(filePath As String, obj As Object)
        Dim jsonStr As String = JsonConvert.SerializeObject(obj)
        Try
            System.IO.File.WriteAllText(filePath, jsonStr)
        Catch
        End Try
    End Sub
#End Region
End Class
