Imports System.Collections.Generic
Imports Spread.Common.DataStore.Models

Namespace DataSourceEntities
	Public Class StudentCalendarDataSource
		Private _schedule As List(Of ScheduleEntry)
		Private _assignments As List(Of Assignment)

		Public Shared ReadOnly Property Subjects() As List(Of String)
		Public Shared ReadOnly Property AssignmentTypes() As List(Of String)

		Private Property Schedule() As List(Of ScheduleEntry)
			Get
				If _schedule Is Nothing Then
					_schedule = New List(Of ScheduleEntry)()
				End If
				Return _schedule
			End Get
			Set
				_schedule = value
			End Set
		End Property

		Private Property Assignments() As List(Of Assignment)
			Get
				If _assignments Is Nothing Then
					_assignments = New List(Of Assignment)()
				End If
				Return _assignments
			End Get
			Set
				_assignments = value
			End Set
		End Property

		Shared Sub New()
			Subjects = New List(Of String)() From { _
				"English", _
				"History", _
				"Physics", _
				"Chemistry", _
				"Maths", _
				"Programming", _
				"Psychology", _
				"French", _
				"Art" _
			}
			AssignmentTypes = New List(Of String)() From { _
				"Test", _
				"Homework" _
			}
		End Sub

		Private Function UpdatedAssignments(year As Integer) As List(Of Assignment)
			Assignments.ForEach(Function(assignment) InlineAssignHelper(assignment.[Date], New DateTime(year, assignment.[Date].Month, assignment.[Date].Day)))
			Return Assignments
		End Function

		Public Function GetSchedule() As List(Of ScheduleEntry)
			If Schedule.Count <> 50 Then
				_schedule = New List(Of ScheduleEntry)()
			End If
			Dim random As New Random()

			For i As Integer = 0 To 49
				Dim entry As New ScheduleEntry(Subjects(random.[Next](0, Subjects.Count)), CType(i Mod 5 + 1, DayOfWeek), 8 + i \ 5)
				Schedule.Add(entry)
			Next

			Return Schedule
		End Function

		Public Function GetAssignments(year As Integer) As List(Of Assignment)
			If Assignments.Count <> 0 Then
				Return UpdatedAssignments(year)
			End If

			Dim random As New Random()

			Dim [date] As DateTime = New DateTime(year, 1, 1).AddDays(random.[Next](0, 8))
			Dim endDate As New DateTime(year, 12, 31)
			While [date] <= endDate
				Dim assignment As New Assignment([date], [String].Join(": ", Subjects(random.[Next](0, Subjects.Count)), AssignmentTypes(random.[Next](0, AssignmentTypes.Count))))
				Assignments.Add(assignment)

				[date] = [date].AddDays(random.[Next](4, 12))
			End While

			Return Assignments
		End Function
		Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
			target = value
			Return value
		End Function
	End Class
End Namespace
