


Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Data
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports Spread.Common.Features.Spread.Common.Features.Sample
Imports Spread.Common.Features.Spread.Common.Features.Utilities

Namespace Spread.ControlExplorer.Components
    Partial Public Class Sidebar
        Inherits UserControl
        Public Delegate Sub SidebarNavigationHandler(sender As Object, e As SidebarNavigationEventArgs)
        Public Event OnSidebarNavigate As SidebarNavigationHandler

        Private m_dataSource As New List(Of Common.Features.Sample.BaseSample)()
        Public Property DataSource() As List(Of Common.Features.Sample.BaseSample)
            Get
                Return m_dataSource
            End Get
            Set
                m_dataSource = Value
                UpdateTreeview()
            End Set
        End Property

        Private Sub UpdateTreeview()
            StackTreeControl.ImageList = New Dictionary(Of String, Image)()
            StackTreeControl.ImageList.Add("Quickstart", My.Resources.fighter_jet_silhouette)
            StackTreeControl.ImageList.Add("Home", My.Resources.home)
            StackTreeControl.ImageList.Add("Plus", My.Resources.plus_symbol_in_a_rounded_black_square)
            StackTreeControl.ImageList.Add("Minus", My.Resources.minus_sign_inside_a_black_rounded_square_shape)
            _tv.Nodes.Clear()
            Dim home As Object = _tv.Nodes.Add("Home", "Home", "Home")
            Dim quickStart As Object = _tv.Nodes.Add("Quickstart", "Quickstart", "Quickstart")
            Dim examplesNode As Object = _tv.Nodes.Add("Examples", "Showcase", "Plus", "Minus")
            Dim featuresNode As Object = _tv.Nodes.Add("Features", "Features", "Plus", "Minus")

            home.Tag = m_dataSource.FirstOrDefault(Function(d) d.Title = "Home")
            quickStart.Tag = m_dataSource.FirstOrDefault(Function(d) d.Title = "Quickstart")
            examplesNode.Tag = New ExampleSample() With {
                .Title = "Showcase",
                .IsGroup = True
            }

            For Each ds As Object In m_dataSource.Where(Function(d) d.NotASample = False)
                If GetType(ExampleSample) Is ds.[GetType]() Then
                    Dim node As Object = examplesNode.Nodes.Add(TryCast(ds, ExampleSample).UniqueName, TryCast(ds, ExampleSample).Title)
                    node.Tag = ds
                    node.IsSearchable = True
                    node.SearchString = TryCast(ds, ExampleSample).Title + vbLf & TryCast(ds, ExampleSample).Description.FromRTFToStrnig()
                    node.Status = ds.Status
                ElseIf GetType(FeatureSample) Is ds.[GetType]() Then
                    Dim featureTree() As Object = TryCast(ds, FeatureSample).FeatureName.Split(","c)
                    Dim parent As Object = featuresNode
                    For Each feature As Object In featureTree
                        Dim fNode As Object = parent.FindNode(feature)
                        If fNode Is Nothing Then
                            fNode = parent.Nodes.Add(feature, feature)
                            fNode.CollapsedImage = My.Resources.arrowhead_pointing_to_the_right
                            fNode.ExpandedImage = My.Resources.sort_down
                            fNode.Tag = New Common.Features.Sample.BaseSample() With {
                                .Title = TryCast(ds, Common.Features.Sample.FeatureSample).FeatureName,
                                .IsGroup = True
                            }
                        End If
                        parent = fNode
                    Next
                    Dim featureNode As Object = featuresNode.FindNode(featureTree.Last(), True)
                    Dim node As Object = featureNode.Nodes.Add(TryCast(ds, Common.Features.Sample.FeatureSample).UniqueName, TryCast(ds, Common.Features.Sample.FeatureSample).Title)
                    node.Tag = ds
                    node.IsSearchable = True
                    node.SearchString = (TryCast(ds, Common.Features.Sample.FeatureSample).Title + vbLf & TryCast(ds, FeatureSample).Description.FromRTFToStrnig() & vbLf) + TryCast(ds, FeatureSample).FeatureName
                    node.Status = ds.Status
                    If (String.IsNullOrEmpty(parent.Status) AndAlso Not String.IsNullOrEmpty(ds.Status)) Then
                      parent.Status = node.Status
                    End If
              End If
            Next

            _tv.SelectedNode = _tv.Nodes(0)
        End Sub

        Public Sub New()
            InitializeComponent()
        End Sub

        Public Sub SetSelectedSample(sample As Common.Features.Sample.BaseSample)
            If sample IsNot Nothing Then
                _tv.SelectedNode = _tv.FindNode(sample.UniqueName, True)
            End If
        End Sub

        Private Sub Sidebar_Load(sender As Object, e As EventArgs)
            Me.textBoxEx1.Refresh()
        End Sub

        Private Sub textBoxEx1_TextChanged(sender As Object, e As EventArgs)
            Me._tv.FilterString = Me.textBoxEx1.Text
        End Sub

        Private Sub _tv_StackNodeClick(sender As Object, e As StackNodeEventArgs)
            RaiseEvent OnSidebarNavigate(Me, New SidebarNavigationEventArgs(TryCast(e.Node.Tag, Common.Features.Sample.BaseSample)))
        End Sub
    End Class

    Public Class SidebarNavigationEventArgs
        Inherits EventArgs
        Public Property Sample() As Common.Features.Sample.BaseSample
            Get
                Return m_Sample
            End Get
            Set
                m_Sample = Value
            End Set
        End Property
        Private m_Sample As Common.Features.Sample.BaseSample

        Public Sub New(sample As Common.Features.Sample.BaseSample)
            Me.Sample = sample
        End Sub
    End Class
End Namespace
