﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Data
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports Spread.Common.Features.Spread.Common.Features.Utilities

Namespace Spread.ControlExplorer.Components

    Public Delegate Sub StackNodeEventHandler(ByVal sender As Object, ByVal e As StackNodeEventArgs)

    Partial Public Class StackNodeControl
        Inherits UserControl

        Public Event StackNodeClick(ByVal sender As Object, ByVal e As StackNodeEventArgs)
        Private _selectionColor As Color = Color.FromArgb(241, 241, 241)
        Private _collapsedImage As Image
        Private _nodes As StackedNodeCollection
        Private _level As Integer
        Private isExpanded1 As Boolean, showImage1 As Boolean = True

        'Public Property SelectionColor As Color
        '    Get
        '        Return _selectionColor
        '    End Get
        '    Set(ByVal value As Color)
        '        _selectionColor = value
        '        lblText.BackColor = CSharpImpl.__Assign(mainPanel.BackColor, CSharpImpl.__Assign(pnlPB.BackColor, CSharpImpl.__Assign(pbMain.BackColor, value)))
        '    End Set
        'End Property
        Public Property SelectionColor As Color
            Get
                Return _selectionColor
            End Get
            Set
                _selectionColor = Value
                pbMain.BackColor = Value
                pnlPB.BackColor = Value
                mainPanel.BackColor = Value
                lblText.BackColor = Value
            End Set
        End Property

        Public Property HoverBackColor As Color
        Public Property HoverForeColor As Color
        Public Property ExpandedImage As Image

        Public Property CollapsedImage As Image
            Get
                Return Me._collapsedImage
            End Get
            Set(ByVal value As Image)
                _collapsedImage = value
                If Not Me.isExpanded1 Then pbMain.Image = value
            End Set
        End Property

        Private _ShouldBeSeen As Boolean = True

        Public Property ShouldBeSeen As Boolean
            Get
                Return _ShouldBeSeen
            End Get
            Set(ByVal value As Boolean)

                If _ShouldBeSeen <> value Then
                    _ShouldBeSeen = value
                End If

                OnNodeVisibleChange()
            End Set
        End Property

        Private _IsVisible As Boolean = False

        Public Property IsVisible As Boolean
            Get
                Return _IsVisible
            End Get
            Set(ByVal value As Boolean)

                If _IsVisible <> value Then
                    _IsVisible = value
                End If

                OnNodeVisibleChange()
            End Set
        End Property

        Public Property ShowImage As Boolean
            Get
                Return ShowImage
            End Get
            Set(ByVal value As Boolean)
                showImage1 = value
                pnlPB.Visible = value

                If value = False Then
                    mainPanel.Padding = New Padding(5 + _level * 8, 0, 0, 0)
                Else
                    mainPanel.Padding = New Padding(_level * 8, 0, 0, 0)
                End If
            End Set
        End Property

        Public ReadOnly Property Nodes As StackedNodeCollection
            Get

                If _nodes Is Nothing Then
                    _nodes = New StackedNodeCollection()
                    AddHandler _nodes.NodeAdded, AddressOf OnNodeAdded
                End If

                Return _nodes
            End Get
        End Property

        Public Property StackedTree As StackTreeControl
        Public Property ParentNode As StackNodeControl
        Public Property Key As String

        Public Property Status As String
            Get
                Return lblStatus.Text
            End Get
            Set(ByVal value As String)
                lblStatus.Text = value
            End Set
        End Property

        Public Property Level As Integer
            Get
                Return Me._level
            End Get
            Friend Set(ByVal value As Integer)
                Me._level = value
                Me.mainPanel.Padding = New Padding(_level * 8, 0, 0, 0)
            End Set
        End Property

        Public Property IsSearchable As Boolean = False
        Public Property SearchString As String = Nothing

        Public Property IsExpanded As Boolean
            Get
                Return Me.isExpanded1
            End Get
            Set(ByVal value As Boolean)

                If isExpanded1 <> value Then
                    Me.isExpanded1 = value
                    PerformExpandCollapse()
                End If
            End Set
        End Property

        Public Sub New()
            InitializeComponent()
            AddHandler Me.MouseEnter, AddressOf StackNodeControl_MouseEnter
            AddHandler lblText.MouseEnter, AddressOf StackNodeControl_MouseEnter
            AddHandler mainPanel.MouseEnter, AddressOf StackNodeControl_MouseEnter
            AddHandler pbMain.MouseEnter, AddressOf StackNodeControl_MouseEnter
            AddHandler Me.MouseLeave, AddressOf StackNodeControl_MouseLeave
            AddHandler lblText.MouseLeave, AddressOf StackNodeControl_MouseLeave
            AddHandler mainPanel.MouseLeave, AddressOf StackNodeControl_MouseLeave
            AddHandler pbMain.MouseLeave, AddressOf StackNodeControl_MouseLeave
            AddHandler lblText.Click, Sub(s, e) Me.OnNodeClicked(Me)
            AddHandler mainPanel.Click, Sub(s, e) Me.OnNodeClicked(Me)
            AddHandler pbMain.Click, Sub(s, e) Me.OnNodeClicked(Me)
            Me.Height = 40
            Me.IsVisible = False
        End Sub

        Public Sub OnNodeClicked(ByVal node As StackNodeControl)
            StateChangeRequest(Nothing, Nothing)
            RaiseEvent StackNodeClick(Me.StackedTree, New StackNodeEventArgs(node))
        End Sub

        Private Sub PerformExpandCollapse()
            StackedTree.SuspendDrawing()
            pbMain.Image = If(Me.IsExpanded, ExpandedImage, CollapsedImage)
            pbMain.Refresh()

            For Each child As StackNodeControl In Me.Nodes
                child.IsVisible = IsExpanded
            Next

            UpdateHeight()
            StackedTree.ResumeDrawing()
        End Sub

        Friend Sub UpdateHeight()
            Me.Height = GetHeight(Me)
            Dim pNode As StackNodeControl = Me.ParentNode

            While pNode IsNot Nothing
                pNode.UpdateHeight()
                pNode = pNode.ParentNode
            End While
        End Sub

        Private Function GetHeight(ByVal node As StackNodeControl) As Integer
            If Not node.IsVisible OrElse Not node.ShouldBeSeen Then Return 0
            Dim totalHeight As Integer = 40

            If Not node.IsExpanded Then
                Return totalHeight
            Else

                For Each child As StackNodeControl In node.Nodes
                    totalHeight += GetHeight(child)
                Next

                Return totalHeight
            End If
        End Function

        Public Function Has(ByVal Key As String) As Boolean
            Return Nodes.Any(Function(c) c.Key = Key)
        End Function

        Public Sub Expand(ByVal Optional expandAll As Boolean = False)
            If expandAll Then

                For Each child As StackNodeControl In Me.Nodes
                    child.Expand(expandAll)
                Next
            End If

            Me.IsExpanded = True
        End Sub

        Public Sub Collapse(ByVal Optional collapseAll As Boolean = False)
            If collapseAll Then

                For Each child As StackNodeControl In Me.Nodes
                    child.Collapse(collapseAll)
                Next
            End If

            Me.IsExpanded = False
        End Sub

        Public Function FindNode(ByVal key As String, ByVal Optional searchAllChild As Boolean = True) As StackNodeControl
            Dim n As StackNodeControl = Nothing

            For Each node As StackNodeControl In Me.Nodes

                If node.Key = key Then
                    n = node
                Else
                    n = node.FindNode(key)
                End If

                If n Is Nothing Then
                    Continue For
                Else
                    Exit For
                End If
            Next

            Return n
        End Function

        Public Overrides Function ToString() As String
            Return lblText.Text
        End Function

        Protected Overrides Sub OnTextChanged(ByVal e As EventArgs)
            MyBase.OnTextChanged(e)
            Me.lblText.Text = Me.Text
        End Sub

        Private Sub OnNodeAdded(ByVal sender As Object, ByVal e As StackNodeEventArgs)
            Dim node As StackNodeControl
            node = e.Node
            node.HoverBackColor = Me.HoverBackColor
            node.HoverForeColor = Me.HoverForeColor
            node.ForeColor = Me.ForeColor
            node.BackColor = Me.BackColor
            node.Level = Me.Level + 1
            node.StackedTree = Me.StackedTree
            node.Width = Me.Width - mainPanel.Padding.Left
            node.ParentNode = Me
            AddHandler node.StackNodeClick, Sub(s, ev) Me.OnNodeClicked(ev)
            Me.childPanel.Controls.Add(node)
        End Sub

        Private Sub OnNodeClicked(ByVal e As StackNodeEventArgs)
            RaiseEvent StackNodeClick(Me, e)
        End Sub

        Private Sub StackNodeControl_MouseLeave(ByVal sender As Object, ByVal e As EventArgs)
            Me.Cursor = Cursors.[Default]
        End Sub

        Private Sub StackNodeControl_MouseEnter(ByVal sender As Object, ByVal e As EventArgs)
            Me.Cursor = Cursors.Hand
        End Sub

        Private Sub StateChangeRequest(ByVal sender As Object, ByVal e As EventArgs)
            Me.IsExpanded = Not Me.IsExpanded
        End Sub

        Private Sub OnNodeVisibleChange()
            Me.Visible = ShouldBeSeen AndAlso IsVisible
            UpdateHeight()
        End Sub

        Private Class CSharpImpl
            <Obsolete("Please refactor calling code to use normal Visual Basic assignment")>
            Shared Function __Assign(Of T)(ByRef target As T, value As T) As T
                target = value
                Return value
            End Function
        End Class
    End Class

    Public Class StackNodeEventArgs
        Inherits EventArgs

        Public Node As StackNodeControl

        Public Sub New(ByVal node As StackNodeControl)
            Me.Node = node
        End Sub
    End Class
End Namespace
