﻿Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms

Namespace Spread.ControlExplorer.Components
    Public Class StackedNodeCollection
        Implements IList(Of StackNodeControl), IEnumerable(Of StackNodeControl)
        Friend Event NodeAdded As StackNodeEventHandler
        Private _nodes As List(Of StackNodeControl)

        Friend Sub New()
            _nodes = New List(Of StackNodeControl)()
        End Sub
        Private Sub OnCollectionModified(ByVal node As StackNodeControl)
            RaiseEvent NodeAdded(Me, New StackNodeEventArgs(node))
        End Sub

        Public Function Add(ByVal key As String, ByVal name As String, ByVal collapsedImage As String, ByVal expandedImage As String) As StackNodeControl
            Dim control As StackNodeControl = New StackNodeControl()
            control.ExpandedImage = If(expandedImage IsNot Nothing, StackTreeControl.ImageList(expandedImage), Nothing)
            control.CollapsedImage = If(collapsedImage IsNot Nothing, StackTreeControl.ImageList(collapsedImage), Nothing)
            control.Text = name
            control.Key = key
            control.IsExpanded = False
            _nodes.Add(control)
            OnCollectionModified(control)
            Return control
        End Function
        Public Function Add(ByVal key As String, ByVal name As String, ByVal image As String) As StackNodeControl
            Return Add(key, name, image, image)
        End Function

        Public Function Add(ByVal key As String, ByVal name As String) As StackNodeControl
            Return Add(key, name, Nothing, Nothing)
        End Function

        Public Function Add(ByVal name As String) As StackNodeControl
            Return Add(name, name, Nothing, Nothing)
        End Function
        Default Public Property Item(index As Integer) As StackNodeControl Implements IList(Of StackNodeControl).Item
            Get
                Return _nodes(index)
            End Get
            Set(ByVal value As StackNodeControl)
                _nodes(index) = value
            End Set
        End Property

        Public ReadOnly Property Count As Integer Implements ICollection(Of StackNodeControl).Count
            Get
                Return _nodes.Count
            End Get
        End Property

        Public ReadOnly Property IsReadOnly As Boolean Implements ICollection(Of StackNodeControl).IsReadOnly
            Get
                Return True
            End Get
        End Property

        Public Sub Insert(index As Integer, item As StackNodeControl) Implements IList(Of StackNodeControl).Insert
            _nodes.Insert(index, item)
        End Sub

        Public Sub RemoveAt(index As Integer) Implements IList(Of StackNodeControl).RemoveAt
            _nodes.RemoveAt(index)
        End Sub

        Public Sub Add(item As StackNodeControl) Implements ICollection(Of StackNodeControl).Add
            _nodes.Add(item)
            OnCollectionModified(item)
        End Sub

        Public Sub Clear() Implements ICollection(Of StackNodeControl).Clear
            _nodes.Clear()
        End Sub

        Public Sub CopyTo(array() As StackNodeControl, arrayIndex As Integer) Implements ICollection(Of StackNodeControl).CopyTo
            _nodes.CopyTo(array, arrayIndex)
        End Sub

        Public Function IndexOf(item As StackNodeControl) As Integer Implements IList(Of StackNodeControl).IndexOf
            Return _nodes.IndexOf(item)
        End Function

        Public Function Contains(item As StackNodeControl) As Boolean Implements ICollection(Of StackNodeControl).Contains
            Return _nodes.Contains(item)
        End Function

        Public Function Remove(item As StackNodeControl) As Boolean Implements ICollection(Of StackNodeControl).Remove
            Return _nodes.Remove(item)
        End Function

        'Private Iterator Function GetEnumerator() As IEnumerator(Of StackNodeControl)
        '    Dim node As Object
        '    For Each node In _nodes
        '        Yield node
        '    Next
        'End Function
        'Public Function GetEnumerator() As IEnumerator
        '    Return _nodes.GetEnumerator()
        'End Function
        Public Iterator Function GetEnumerator() As IEnumerator(Of StackNodeControl) Implements IEnumerable(Of StackNodeControl).GetEnumerator
            Dim node As Object
            For Each node In _nodes
                Yield CType(node, StackNodeControl)
            Next

        End Function

        Private Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return _nodes.GetEnumerator()

        End Function
    End Class
End Namespace
