﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Data
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports Spread.Common.Features.Spread.Common.Features.Utilities
Imports Spread.Common.Features.Spread.Common.Features.Explorer





Namespace Spread.ControlExplorer.Components
    Partial Public Class StackTreeControl
        Inherits UserControl

        Public Event SelectionChanged As StackNodeEventHandler
        Public Shared Property ImageList As Dictionary(Of String, Image) = New Dictionary(Of String, Image)()
        Private _filterString As String = ""
        Private _selectedNode As StackNodeControl
        Private _nodes As StackedNodeCollection

        Public Property FilterString As String
            Get
                Return _filterString
            End Get
            Set(ByVal value As String)

                If String.IsNullOrEmpty(value) OrElse String.IsNullOrWhiteSpace(value) Then
                    _filterString = ""
                Else
                    _filterString = value
                End If

                Me.SuspendDrawing()

                For Each node As StackNodeControl In Me.Nodes
                    node.Expand(GetFilteredContentOnlyChild(node))
                Next

                Me.ResumeDrawing()
            End Set
        End Property

        Private Function GetFilteredContentOnlyChild(ByVal node As StackNodeControl) As Boolean
            If node.IsSearchable Then

                If node.SearchString IsNot Nothing Then
                    node.ShouldBeSeen = node.SearchString.ToLower().Contains(_filterString.ToLower())
                    Return node.ShouldBeSeen
                End If

                Return True
            Else

                If node.Nodes.Count > 0 Then
                    Dim somethingIsVisble As Boolean = False

                    For Each n As StackNodeControl In node.Nodes
                        somethingIsVisble = somethingIsVisble Or GetFilteredContentOnlyChild(n)
                    Next

                    node.ShouldBeSeen = somethingIsVisble
                    Return node.ShouldBeSeen
                Else
                    Return True
                End If
            End If
        End Function

        Public Property HoverBackColor As Color
        Public Property HoverForeColor As Color

        Public Property SelectedNode As StackNodeControl
            Get
                Return _selectedNode
            End Get
            Set(ByVal value As StackNodeControl)

                If value IsNot Nothing AndAlso _selectedNode IsNot value Then

                    If _selectedNode IsNot Nothing Then
                        _selectedNode.SelectionColor = Color.White
                    End If

                    _selectedNode = value
                    _selectedNode.SelectionColor = Color.FromArgb(241, 241, 241)
                    RaiseEvent SelectionChanged(Me, New StackNodeEventArgs(_selectedNode))
                End If
            End Set
        End Property

        Public ReadOnly Property Nodes As StackedNodeCollection
            Get

                If _nodes Is Nothing Then
                    _nodes = New StackedNodeCollection()
                    AddHandler _nodes.NodeAdded, AddressOf OnNodeAdded
                    AddHandler _nodes.NodeAdded, AddressOf OnNodeAdded
                End If

                Return _nodes
            End Get
        End Property

        Private Sub OnNodeAdded(ByVal sender As Object, ByVal e As StackNodeEventArgs)

            Dim node As StackNodeControl
            node = e.Node
            node.HoverBackColor = Me.HoverBackColor
            node.HoverForeColor = Me.HoverForeColor
            node.ForeColor = Me.ForeColor
            node.BackColor = Me.BackColor
            node.Level = 0
            node.Width = Me.Width
            node.StackedTree = Me
            node.ParentNode = Nothing
            node.IsVisible = True
            Me.pnlMain.Controls.Add(node)

            AddHandler node.StackNodeClick, Sub(s, ev) Me.OnStackNodeClick(ev)
        End Sub

        Public Sub New()
            InitializeComponent()
        End Sub

        Public Function FindNode(ByVal key As String, ByVal Optional searchAllChild As Boolean = True) As StackNodeControl
            Dim n As StackNodeControl = Nothing

            For Each node As StackNodeControl In Me.Nodes

                If node.Key = key Then
                    n = node
                Else
                    n = node.FindNode(key)
                End If

                If n Is Nothing Then
                    Continue For
                Else
                    Exit For
                End If
            Next

            Return n
        End Function

        Public Sub FilterNode(ByVal filterText As String)
            For Each node As StackNodeControl In Me.Nodes

                If node.IsSearchable Then
                End If
            Next
        End Sub

        Public Sub OnStackNodeClick(ByVal e As StackNodeEventArgs)
            Me.SelectedNode = e.Node
        End Sub
    End Class
End Namespace
