Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Windows.Forms
Imports System.Runtime.CompilerServices
Imports System.Runtime.InteropServices

Namespace Spread.Common.Features.Utilities
    Module Extensions
        <Extension()>
        Function AllIndexesOf(ByVal str As String, ByVal value As String) As List(Of Integer)
            If String.IsNullOrEmpty(value) Then Return New List(Of Integer)()
            Dim indexes As List(Of Integer) = New List(Of Integer)()
            Dim index As Integer = 0

            While True
                index = str.IndexOf(value, index)
                If index = -1 Then Return indexes
                indexes.Add(index)
                index += value.Length
            End While
        End Function

        <Extension()>
        Function GetNext(Of T)(ByVal list As IEnumerable(Of T), ByVal current As T) As T
            Try
                If current Is Nothing OrElse current.Equals(list.LastOrDefault()) Then Return list.FirstOrDefault()
                Return list.SkipWhile(Function(x) Not x.Equals(current)).Skip(1).First()
            Catch
                Return Nothing
            End Try
        End Function

        <Extension()>
        Function GetPrevious(Of T)(ByVal list As IEnumerable(Of T), ByVal current As T) As T
            Try
                If current Is Nothing OrElse current.Equals(list.FirstOrDefault()) Then Return list.LastOrDefault()
                Return list.TakeWhile(Function(x) Not x.Equals(current)).Last()
            Catch
                Return Nothing
            End Try
        End Function

        <Extension()>
        Function FromRTFToStrnig(ByVal rtf As String) As String
            Try
                Return New RichTextBox() With {
                    .Rtf = rtf
                }.Text
            Catch
                Return String.Empty
            End Try
        End Function

        <Extension()>
        Function NormalizeRtf(ByVal source As String) As String
            Dim result As Object = ("{\rtf1\ansi\deff0{\colortbl;\red100\green100\blue100;\red255\green0\blue0;\red0\green0\blue255;}" & source & " }").Replace("[b]", "\b ").Replace("[/b]", "\b0 ").Replace("[color]", "\cf1 ").Replace("[/color]", "\cf0 ").Replace(vbCrLf, "\line ").Replace(vbLf, "\line ")
            Return result.ToString()

        End Function

        <Extension()>
        Sub FormatRfToCS(ByVal rtb As RichTextBox)
            Dim reservedWords As String() = New String() {"abstract", "as", "base", "bool", "break", "byte", "case", "catch", "char", "checked", "class", "const", "continue", "decimal", "default", "delegate", "do", "double", "dynamic", "else", "enum", "event", "explicit", "extern", "false", "finally", "fixed", "float", "for", "foreach", "goto", "if", "implicit", "in", "int", "interface", "internal", "is", "lock", "long", "namespace", "new", "null", "object", "operator", "out", "override", "params", "partial", "private", "protected", "public", "readonly", "ref", "return", "sbyte", "sealed", "short", "sizeof", "stackalloc", "static", "string", "struct", "switch", "this", "throw", "true", "try", "typeof", "uint", "ulong", "unchecked", "unsafe", "ushort", "using", "using static", "var", "virtual", "void", "volatile", "while"}
            Dim reservedWord As String
            For Each reservedWord In reservedWords

                For Each foundAt As Integer In rtb.Text.AllIndexesOf(reservedWord)
                    Dim validBegin As Boolean = False
                    Dim validEnd As Boolean = False

                    If foundAt > 0 Then
                        Dim beforeString As String = rtb.Text(foundAt - 1).ToString()

                        If Not Regex.IsMatch(beforeString, "^[a-zA-Z0-9_]+$") Then
                            validBegin = True
                        End If
                    Else
                        validBegin = True
                    End If

                    If foundAt + reservedWord.Length < rtb.Text.Length Then
                        Dim afterString As String = rtb.Text(foundAt + reservedWord.Length).ToString()

                        If Not Regex.IsMatch(afterString, "^[a-zA-Z0-9_]+$") Then
                            validEnd = True
                        End If
                    Else
                        validEnd = True
                    End If

                    If validEnd AndAlso validBegin Then
                        rtb.[Select](foundAt, reservedWord.Length)
                        rtb.SelectionColor = System.Drawing.Color.Blue
                    End If
                Next
            Next

            Dim commentPattern As String = "((?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/)|(?:\/\/.*))"

            For Each match As Match In Regex.Matches(rtb.Text, commentPattern)
                rtb.[Select](match.Index, match.Length)
                rtb.SelectionColor = System.Drawing.Color.DarkGreen
            Next

            rtb.[Select](0, 0)
            rtb.ForeColor = System.Drawing.Color.Black
            rtb.BackColor = System.Drawing.Color.White
        End Sub

        <System.Runtime.InteropServices.DllImport("user32.dll")>
        Function SendMessage(ByVal hWnd As IntPtr, ByVal wMsg As Int32, ByVal wParam As Boolean, ByVal lParam As Int32) As Integer
        End Function
        Const WM_SETREDRAW As Integer = 11

        <Extension()>
        Sub SuspendDrawing(ByVal control As System.Windows.Forms.Control)
            SendMessage(control.Handle, WM_SETREDRAW, False, 0)
        End Sub

        <Extension()>
        Sub ResumeDrawing(ByVal control As System.Windows.Forms.Control)
            SendMessage(control.Handle, WM_SETREDRAW, True, 0)
            control.Refresh()
        End Sub
    End Module
End Namespace
