
Imports Spread.Common.Features.Spread.Common.Features.Sample
Imports Spread.Common.Features.Spread.Common.Features.Utilities
Imports Spread.Common.Features.Spread.ControlExplorer.Components

Imports System.Collections.Generic
Imports System.Drawing
Imports System.IO
Imports System.Linq
Imports System.Reflection
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Forms


Namespace Spread.Common.Features
    Public Class Loader
        Private Shared Manager As Manager
        Private Shared Current As BaseSample
        Private Shared tileControl As TileControl
        Private Shared host As SampleHost
        Private Shared sideBar As Sidebar
        Private Shared ReadOnly _locker As New Object()

        Public Shared Sub LoadConfig(path As String, sidebar__1 As Sidebar, header1 As Header, tile As TileControl, sampleHost As SampleHost)
            Manager = New Manager(path)
            tileControl = tile
            host = sampleHost
            sideBar = sidebar__1
            sideBar.DataSource = Manager.Samples
            header1.Title = Manager.Header1
            header1.DownloadLink = Manager.DownloadLink
            AddHandler header1.OnHeaderNavigate, Sub(s, e)
                                                     Dim sampleToNavigate As BaseSample = Nothing
                                                     Select Case e.Direction
                                                         Case NavigationDirectionEnum.First
                                                             sampleToNavigate = Manager.Samples.First()
                                                             Exit Select
                                                         Case NavigationDirectionEnum.Last
                                                             sampleToNavigate = Manager.Samples.Last()
                                                             Exit Select
                                                         Case NavigationDirectionEnum.[Next]
                                                             sampleToNavigate = Manager.Samples.GetNext(Current)
                                                             Exit Select
                                                         Case NavigationDirectionEnum.Previous
                                                             sampleToNavigate = Manager.Samples.GetPrevious(Current)
                                                             Exit Select
                                                     End Select
                                                     sideBar.SetSelectedSample(sampleToNavigate)
                                                     Current = sampleToNavigate

                                                 End Sub
        End Sub

        Public Shared Function GetSampleType(sample As BaseSample) As Type
            Dim assembly__1 As Assembly = Nothing
            If Not String.IsNullOrWhiteSpace(sample.ExeName) Then
                Try
                    assembly__1 = Assembly.LoadFrom(sample.ExeName)
                Catch
                    assembly__1 = Assembly.GetExecutingAssembly()

                End Try
            Else
                assembly__1 = Assembly.GetEntryAssembly()
            End If
            Dim types() As Type = assembly__1.GetTypes()
            Return types.Where(Function(type) type.IsSubclassOf(GetType(BaseSampleControl)) AndAlso type.Name.Equals(sample.ClassName)).FirstOrDefault()
        End Function

        Public Shared Sub ShowSample(sample As BaseSample)
            Dim t As Object = GetSampleType(sample)
            If t IsNot Nothing Then
                Dim obj As Object = TryCast(Activator.CreateInstance(t), BaseSampleControl)
                If obj IsNot Nothing Then
                    host.Description = sample.Description
                    host.Code = sample.Code
                    Current = sample
                    obj.Dock = DockStyle.Fill
                    host.Sample = CType(obj, Control)
                    host.Visible = True
                    tileControl.Visible = False
                End If
            End If
        End Sub

        Public Shared Sub ShowTiles(baseSample As BaseSample)
            tileControl.Groups.Clear()
            Dim group As New TileGroup(baseSample.Title)
            Dim samples As New List(Of BaseSample)()
            If TypeOf baseSample Is ExampleSample Then
                samples.AddRange(Manager.Samples.OfType(Of ExampleSample)())
            Else
                samples.AddRange(Manager.Samples.OfType(Of FeatureSample)().Where(Function(s) s.FeatureName.StartsWith(baseSample.Title)))
            End If

            Dim imagesDir As Object = Path.Combine(Directory.GetCurrentDirectory(), "Images")
            For Each sample As BaseSample In samples
                Dim img As Image = Nothing
                Try
                    Dim exe As Object = sample.ExeName.Replace(".exe", String.Empty)
                    Dim imageName As String = String.Format("{0}.{1}.gif", exe, sample.ClassName)
                    Dim fileName As String = Path.Combine(CType(imagesDir, String), imageName)
                    img = Image.FromFile(fileName)
                Catch generatedExceptionName As Exception
                    img = My.Resources.Spread_WaterMark
                    img.Tag = "PlaceHolder"
                End Try
                Dim tile As New TileView() With {
                .Text = sample.Title.ToString(),
                .Image = img,
                .Tag = sample
            }
                AddHandler tile.Click, Sub(s, e) sideBar.SetSelectedSample(sample)
                group.Tiles.Add(tile)
            Next
            tileControl.Groups.Add(group)
            host.Visible = False
            tileControl.Visible = True
            SyncLock _locker
                UpdateImages()
            End SyncLock
        End Sub

        Private Shared Async Function UpdateImages() As Task
            For Each group As TileGroup In tileControl.Groups
                For Each tile As TileView In group.Tiles.Where(Function(t) t.Image.Tag IsNot Nothing)
                    tile.Image = Await GetImage(TryCast(tile.Tag, BaseSample))
                Next
            Next
        End Function

        Private Shared _imgCache As New Dictionary(Of String, Image)()

        Private Shared Async Function GetImage(sample As BaseSample) As Task(Of Image)
            Dim image As Image = Nothing
            Await Task.Factory.StartNew(Sub()
                                            Try
                                                image = _imgCache.FirstOrDefault(Function(img) img.Key = sample.Title).Value
                                                If image Is Nothing Then
                                                    Dim type As Object = GetSampleType(sample)
                                                    If type IsNot Nothing Then
                                                        Dim obj As Object = TryCast(Activator.CreateInstance(type), Spread.Common.Features.Sample.BaseSampleControl)
                                                        image = obj.GetImage()
                                                    End If
                                                End If
                                            Catch ex As Exception
                                                System.Diagnostics.Debug.WriteLine(ex.Message)
                                            Finally
                                                If image Is Nothing Then
                                                    image = My.Resources.Spread_WaterMark
                                                End If

                                                If Not _imgCache.ContainsKey(sample.Title) Then
                                                    _imgCache.Add(sample.Title, image)
                                                End If
                                            End Try

                                        End Sub)
            Return image
        End Function
    End Class
End Namespace
