Imports System.Collections.Generic
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms

Namespace Spread.Common.Features.Explorer.CustomControls
    Public Class RoundButton
        Inherits Button
        Private m_cornerRadius As Integer = 1
        Public Property CornerRadius() As Integer
            Get
                Return m_cornerRadius
            End Get
            Set
                Me.m_cornerRadius = Value
                Me.Refresh()
            End Set
        End Property
        Public Property OldCursor() As Cursor
            Get
                Return m_OldCursor
            End Get
            Private Set
                m_OldCursor = Value
            End Set
        End Property
        Private m_OldCursor As Cursor

        Protected Overrides Sub OnMouseEnter(e As EventArgs)
            OldCursor = Me.Cursor
            MyBase.OnMouseEnter(e)
            Me.Cursor = Cursors.Hand
        End Sub

        Protected Overrides Sub OnMouseLeave(e As EventArgs)
            Me.Cursor = OldCursor
            MyBase.OnMouseLeave(e)
        End Sub

        Protected Overrides Sub OnPaint(e As System.Windows.Forms.PaintEventArgs)
            MyBase.OnPaint(e)
            If m_cornerRadius > 0 Then
                Dim Rect As New RectangleF(0, 0, Me.Width, Me.Height)
                Dim GraphPath As GraphicsPath = GetRoundPath(Rect, Me.CornerRadius)

                Me.Region = New System.Drawing.Region(GraphPath)
            End If


        End Sub

        Private Function GetRoundPath(Rect As RectangleF, radius As Integer) As GraphicsPath
            Dim r2 As Single = radius / 2.0F
            Dim GraphPath As New GraphicsPath()

            GraphPath.AddArc(Rect.X, Rect.Y, radius, radius, 180, 90)

            GraphPath.AddLine(Rect.X + r2, Rect.Y, Rect.Width - r2, Rect.Y)

            GraphPath.AddArc(Rect.X + Rect.Width - radius, Rect.Y, radius, radius, 270, 90)

            GraphPath.AddLine(Rect.Width, Rect.Y + r2, Rect.Width, Rect.Height - r2)

            GraphPath.AddArc(Rect.X + Rect.Width - radius, Rect.Y + Rect.Height - radius, radius, radius, 0, 90)

            GraphPath.AddLine(Rect.Width - r2, Rect.Height, Rect.X + r2, Rect.Height)

            GraphPath.AddArc(Rect.X, Rect.Y + Rect.Height - radius, radius, radius, 90, 90)

            GraphPath.AddLine(Rect.X, Rect.Height - r2, Rect.X, Rect.Y + r2)

            GraphPath.CloseFigure()

            Return GraphPath
        End Function

    End Class
End Namespace
