Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Xml.Linq


Namespace Spread.Common.Features.Sample
    Public Class Manager
        Public Property Header1() As String
            Get
                Return m_Header
            End Get
            Set
                m_Header = Value
            End Set
        End Property
        Private m_Header As String
        Public Property DownloadLink() As String
            Get
                Return m_DownloadLink
            End Get
            Set
                m_DownloadLink = Value
            End Set
        End Property
        Private m_DownloadLink As String
        Public Samples As New List(Of BaseSample)()

        Public Sub New(path As String)
            Try
                Dim xdoc As XDocument = XDocument.Load(path)

                ' Header
                If xdoc.Root.Elements().Any(Function(e As XElement) e.Name = "header") Then
                    Dim headerElement As XElement = xdoc.Root.Element("header")

                    ' Title
                    If headerElement.Elements().Any(Function(e As XElement) e.Name = "title") Then
                        Dim titleElement As XElement = headerElement.Element("title")
                        Header1 = titleElement.Value.ToString()
                    Else
                        Throw New Exception("`header>title` element is missing")
                    End If

                    ' Download Link
                    If headerElement.Elements().Any(Function(e As XElement) e.Name = "download") Then
                        Dim downloadElement As XElement = headerElement.Element("download")
                        DownloadLink = downloadElement.Value.ToString()

                    Else
                        Throw New Exception("`header>download` element is missing")
                    End If

                    ' Home
                    If headerElement.Elements().Any(Function(e As XElement) e.Name = "home") Then
                        Dim homeElement As XElement = headerElement.Element("home")
                        Dim sample As BaseSample = New BaseSample() With {
                            .Title = "Home",
                            .ClassName = If(homeElement.Attributes().Any(Function(h) h.Name = "host"), homeElement.Attribute("host").Value, Nothing),
                            .NotASample = True
                        }
                        Samples.Add(sample)
                    Else
                        Throw New Exception("`header>home` element is missing")
                    End If

                    ' Quickstart
                    If headerElement.Elements().Any(Function(e As XElement) e.Name = "quick-start") Then
                        Dim downElement As XElement = headerElement.Element("quick-start")

                        Dim sample As BaseSample = New BaseSample() With {
                            .Title = "Quickstart",
                            .ClassName = If(downElement.Attributes().Any(Function(h) h.Name = "host"), downElement.Attribute("host").Value, Nothing),
                            .NotASample = True
                        }
                        Samples.Add(sample)
                    Else
                        Throw New Exception("`header>quick-start` element is missing")
                    End If
                Else
                    Throw New Exception("`header` element is missing")
                End If

                ' Examples
                If xdoc.Root.Elements().Any(Function(e As XElement) e.Name = "examples") Then
                    Dim exElement As XElement = xdoc.Root.Element("examples")
                    For Each example As XElement In exElement.Elements().Where(Function(elem As XElement) elem.Name = "example")
                        Try
                            Dim desc As Object = If(example.Elements().Any(Function(e As XElement) e.Name = "description"), example.Element("description").Value, Nothing)
                            Dim _path As Object = If(example.Elements().Any(Function(e As XElement) e.Name = "description"), If(example.Element("description").Attributes().Any(Function(p) p.Name = "path"), example.Element("description").Attribute("path").Value, Nothing), Nothing)
                            If _path IsNot Nothing Then
                                Try
                                    _path = System.IO.Path.Combine("Descriptions\Examples", _path)
                                    desc = System.IO.File.ReadAllText(_path)
                                Catch
                                End Try
                            End If
                            Dim code As String = If(example.Elements().Any(Function(e As XElement) e.Name = "code"), example.Element("code").Value, Nothing)
                            _path = If(example.Elements().Any(Function(e As XElement) e.Name = "code"), If(example.Element("code").Attributes().Any(Function(p) p.Name = "path"), example.Element("code").Attribute("path").Value, Nothing), Nothing)
                            If _path IsNot Nothing Then
                                Try
                                    _path = System.IO.Path.Combine("Codes\Examples", _path)
                                    code = System.IO.File.ReadAllText(_path)
                                Catch
                                End Try
                            End If
                            Dim sample = New ExampleSample() With {
                                .ExeName = If(example.Attributes().Any(Function(e) e.Name = "exename"), example.Attribute("exename").Value, Nothing),
                                .ClassName = If(example.Attributes().Any(Function(e) e.Name = "className"), example.Attribute("className").Value, Nothing),
                                .Title = If(example.Attributes().Any(Function(e) e.Name = "title"), example.Attribute("title").Value, Nothing),
                                .Status = If(example.Attributes().Any(Function(e) e.Name = "status"), example.Attribute("status").Value, Nothing),
                                .Description = desc,
                                .Code = code
                            }
                            Samples.Add(sample)
                        Catch
                            Throw New Exception("Error parsing `example` element")
                        End Try
                    Next
                Else
                    Throw New Exception("`examples` element is missing")
                End If

                ' Features
                If xdoc.Root.Elements().Any(Function(e As XElement) e.Name = "features") Then
                    Dim featuresElement As XElement = xdoc.Root.Element("features")
                    For Each feature As XElement In featuresElement.Elements().Where(Function(elem) elem.Name = "feature")
                        Try
                            Dim desc As Object = If(feature.Elements().Any(Function(e As XElement) e.Name = "description"), feature.Element("description").Value, Nothing)

                            Dim _path As Object = If(feature.Elements().Any(Function(e As XElement) e.Name = "description"), If(feature.Element("description").Attributes().Any(Function(p) p.Name = "path"), feature.Element("description").Attribute("path").Value, Nothing), Nothing)
                            If _path IsNot Nothing Then
                                Try
                                    _path = System.IO.Path.Combine("Descriptions\Features", _path)
                                    desc = System.IO.File.ReadAllText(_path)
                                Catch
                                End Try
                            End If
                            Dim code As Object = If(feature.Elements().Any(Function(e As XElement) e.Name = "code"), feature.Element("code").Value, Nothing)
                            _path = If(feature.Elements().Any(Function(e As XElement) e.Name = "code"), If(feature.Element("code").Attributes().Any(Function(p) p.Name = "path"), feature.Element("code").Attribute("path").Value, Nothing), Nothing)
                            If _path IsNot Nothing Then
                                Try
                                    _path = System.IO.Path.Combine("Codes\Features", _path)
                                    code = System.IO.File.ReadAllText(_path)
                                Catch
                                End Try
                            End If

                            Dim sample = New FeatureSample() With {
    .ExeName = If(feature.Attributes().Any(Function(e) e.Name = "exename"), feature.Attribute("exename").Value, Nothing),
    .ClassName = If(feature.Attributes().Any(Function(e) e.Name = "className"), feature.Attribute("className").Value, Nothing),
    .Description = desc,
    .Code = code,
    .Title = If(feature.Attributes().Any(Function(e) e.Name = "title"), feature.Attribute("title").Value, Nothing),
    .FeatureName = If(feature.Attributes().Any(Function(e) e.Name = "featurename"), feature.Attribute("featurename").Value, Nothing),
    .Status = If(feature.Attributes().Any(Function(e) e.Name = "status"), feature.Attribute("status").Value, Nothing)
}
                            Samples.Add(sample)
                        Catch
                            Throw New Exception("Error parsing `feature` element")
                        End Try
                    Next
                Else
                    Throw New Exception("`features` element is missing")
                End If
            Catch e As Exception
                System.Windows.Forms.MessageBox.Show("Config.xml's structure is invalid : " & e.Message)
            End Try
        End Sub
    End Class
End Namespace
